
#include <GL/glut.h>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>           /* ULONG_MAX is defined here */
#include <float.h>            /* FLT_MAX is atleast defined here */

#include <time.h>  /* for random seed */

//#include "fracviewer.h"

#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif

/***************************************************************/
/************************* VECTOR JUNK *************************/
/***************************************************************/

typedef struct point3fstruct {
float v[3];
} Point3f;

  /* print vertex to stderr */
void printvert(Point3f *p)
{
  fprintf(stderr, "(%f, %f, %f)\n", p->v[0], p->v[1], p->v[2]);
}

  /* normalizes v */
void normalize(Point3f *p)
{
  GLfloat d = sqrt(p->v[0]*p->v[0] + p->v[1]*p->v[1] + p->v[2]*p->v[2]);

  if (d == 0)
    fprintf(stderr, "Zero length vector in normalize\n");
  else
    p->v[0] /= d; p->v[1] /= d; p->v[2] /= d;
}

  /* calculates a normalized crossproduct to v1, v2 */
void ncrossprod(Point3f *v1,Point3f *v2,Point3f *cp)
{
  cp->v[0] = v1->v[1]*v2->v[2] - v1->v[2]*v2->v[1];
  cp->v[1] = v1->v[2]*v2->v[0] - v1->v[0]*v2->v[2];
  cp->v[2] = v1->v[0]*v2->v[1] - v1->v[1]*v2->v[0];
  normalize(cp);
}

  /* calculates normal to the triangle designated by v1, v2, v3 */
void triagnormal(Point3f *v1, Point3f *v2, Point3f *v3, Point3f * norm)
{
  Point3f vec1,vec2;
  vec1.v[0] = v3->v[0] - v1->v[0];  vec2.v[0] = v2->v[0] - v1->v[0];
  vec1.v[1] = v3->v[1] - v1->v[1];  vec2.v[1] = v2->v[1] - v1->v[1];
  vec1.v[2] = v3->v[2] - v1->v[2];  vec2.v[2] = v2->v[2] - v1->v[2];

  ncrossprod(&vec2, &vec1, norm);
}

/***************************************************************/
/************************ MOUNTAIN STUFF ***********************/
/***************************************************************/

void myGLInit(void)
{
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);
  glShadeModel(GL_SMOOTH);

  glFlush();
} 

/***************************************************************/
/************************ GLUT STUFF ***************************/
/***************************************************************/
static void
  Key(unsigned char key, int x, int y)
{
  switch (key) {
  case 27:
    exit(0);
  }
}

void reshape(GLsizei w, GLsizei h)
{
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60.0, (GLdouble)w/h, 0.01, 100);
  gluLookAt(5,5,5,
		0,0,0,
		0,1,0);
  glPushMatrix();
  glMatrixMode(GL_MODELVIEW);

}

void display(void)
{ 
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glFlush();
  glutSwapBuffers();
}

/***************************************************************/
/**************************** MAIN *****************************/
/***************************************************************/

int main(int argc, char** argv)
{
  glutInit(&argc, argv);
  glutInitWindowSize(512, 512);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
  glutCreateWindow("Fractal Planet");

  glutReshapeFunc(reshape);
  glutDisplayFunc(display);
  glutKeyboardFunc(Key);	
  
  myGLInit(); 
  
  glutMainLoop();
  return 0;             /* ANSI C requires main to return int. */
}
